@@ -32,30 +32,27 @@
  */
 
 var COMPONENT = 'atto_table',
-    EDITTEMPLATE = '' +
-        '<form class="{{CSS.FORM}}">' +
-            '<label for="{{elementid}}_atto_table_caption">{{get_string "caption" component}}</label>' +
-            '<input class="{{CSS.CAPTION}} fullwidth" id="{{elementid}}_atto_table_caption" required />' +
-            '<br/>' +
-            '<br/>' +
-            '<label for="{{elementid}}_atto_table_headers" class="sameline">{{get_string "headers" component}}</label>' +
-            '<select class="{{CSS.HEADERS}}" id="{{elementid}}_atto_table_headers">' +
-                '<option value="columns">{{get_string "columns" component}}' + '</option>' +
-                '<option value="rows">{{get_string "rows" component}}' + '</option>' +
-                '<option value="both">{{get_string "both" component}}' + '</option>' +
-            '</select>' +
-            '<br/>' +
-            '<div class="mdl-align">' +
-                '<br/>' +
-                '<button class="submit" type="submit">{{get_string "updatetable" component}}</button>' +
-            '</div>' +
-        '</form>',
+    DEFAULT = {
+        BORDERSTYLE: 'none',
+        BORDERWIDTH: '1'
+    },
+    DIALOGUE = {
+        WIDTH: '480px'
+    },
     TEMPLATE = '' +
         '<form class="{{CSS.FORM}}">' +
             '<label for="{{elementid}}_atto_table_caption">{{get_string "caption" component}}</label>' +
             '<input class="{{CSS.CAPTION}} fullwidth" id="{{elementid}}_atto_table_caption" required />' +
             '<br/>' +
             '<br/>' +
+            '<label for="{{elementid}}_atto_table_captionposition" class="sameline">' +
+            '{{get_string "captionposition" component}}</label>' +
+            '<select class="{{CSS.CAPTIONPOSITION}}" id="{{elementid}}_atto_table_captionposition">' +
+                '<option value=""></option>' +
+                '<option value="top">{{get_string "top" "editor"}}</option>' +
+                '<option value="bottom">{{get_string "bottom" "editor"}}</option>' +
+            '</select>' +
+            '<br/>' +
             '<label for="{{elementid}}_atto_table_headers" class="sameline">{{get_string "headers" component}}</label>' +
             '<select class="{{CSS.HEADERS}}" id="{{elementid}}_atto_table_headers">' +
                 '<option value="columns">{{get_string "columns" component}}' + '</option>' +
@@ -63,32 +60,152 @@ var COMPONENT = 'atto_table',
                 '<option value="both">{{get_string "both" component}}' + '</option>' +
             '</select>' +
             '<br/>' +
-            '<label for="{{elementid}}_atto_table_rows" class="sameline">{{get_string "numberofrows" component}}</label>' +
-            '<input class="{{CSS.ROWS}}" type="number" value="3" id="{{elementid}}_atto_table_rows" size="8" min="1" max="50"/>' +
-            '<br/>' +
-            '<label for="{{elementid}}_atto_table_columns" class="sameline">{{get_string "numberofcolumns" component}}</label>' +
-            '<input class="{{CSS.COLUMNS}}" type="number" value="3" id="{{elementid}}_atto_table_columns" size="8" min="1" max="20"/>' +
-            '<br/>' +
-            '<div class="mdl-align">' +
+            '{{#if nonedit}}' +
+                '<label for="{{elementid}}_atto_table_rows" class="sameline">{{get_string "numberofrows" component}}</label>' +
+                '<input class="{{CSS.ROWS}}" type="number" value="3" ' +
+                'id="{{elementid}}_atto_table_rows" size="8" min="1" max="50"/>' +
                 '<br/>' +
-                '<button class="{{CSS.SUBMIT}}" type="submit">{{get_string "createtable" component}}</button>' +
+                '<label for="{{elementid}}_atto_table_columns" ' +
+                'class="sameline">{{get_string "numberofcolumns" component}}</label>' +
+                '<input class="{{CSS.COLUMNS}}" type="number" value="3" id="{{elementid}}_atto_table_columns"' +
+                'size="8" min="1" max="20"/>' +
+                '<br/>' +
+            '{{/if}}' +
+            '{{#if allowStyling}}' +
+                '<fieldset>' +
+                '<legend class="mdl-align">{{get_string "appearance" component}}</legend>' +
+                '{{#if allowBorders}}' +
+                    '<label for="{{elementid}}_atto_table_borders" class="sameline">{{get_string "borders" component}}</label>' +
+                    '<select name="borders" class="{{CSS.BORDERS}}" id="{{elementid}}_atto_table_borders">' +
+                        '<option value="default">{{get_string "themedefault" component}}' + '</option>' +
+                        '<option value="outer">{{get_string "outer" component}}' + '</option>' +
+                        '<option value="all">{{get_string "all" component}}' + '</option>' +
+                    '</select>' +
+                    '<br>' +
+                    '<label for="{{elementid}}_atto_table_borderstyle" class="sameline">' +
+                    '{{get_string "borderstyles" component}}</label>' +
+                    '<select name="borderstyles" class="{{CSS.BORDERSTYLE}}" id="{{elementid}}_atto_table_borderstyle">' +
+                        '{{#each borderStyles}}' +
+                            '<option value="' + '{{this}}' + '">' + '{{get_string this ../component}}' + '</option>' +
+                        '{{/each}}' +
+                    '</select>' +
+                    '<br>' +
+                    '<label for="{{elementid}}_atto_table_bordersize" class="sameline">' +
+                    '{{get_string "bordersize" component}}</label>' +
+                    '<input name="bordersize" id="{{elementid}}_atto_table_bordersize" class="{{CSS.BORDERSIZE}}"' +
+                    'type="number" value="1" size="8" min="1" max="50"/>' +
+                    '<label style="display: inline-block;">{{CSS.BORDERSIZEUNIT}}</label>' +
+                    '<br>' +
+                    '<label for="{{elementid}}_atto_table_bordercolour" class="sameline">' +
+                    '{{get_string "bordercolour" component}}</label>' +
+                    '<div id="{{elementid}}_atto_table_bordercolour"' +
+                    'class="{{CSS.BORDERCOLOUR}} {{CSS.AVAILABLECOLORS}}" size="1">' +
+                        '<label class="hideborder" for="{{../elementid}}_atto_table_bordercolour_-1"' +
+                        'style="background-color:transparent;color:transparent">' +
+
+                            '<input id="{{../elementid}}_atto_table_bordercolour_-1"' +
+                            'type="radio" name="borderColour" value="none" checked="checked"' +
+                            'title="{{get_string "themedefault" component}}"></input>' +
+
+                            '{{get_string "themedefault" component}}' +
+                        '</label>' +
+                        '{{#each availableColours}}' +
+                            '<label for="{{../elementid}}_atto_table_bordercolour_{{@index}}"' +
+                            'style="background-color:{{this}};color:{{this}}">' +
+
+                                '<input id="{{../elementid}}_atto_table_bordercolour_{{@index}}"' +
+                                'type="radio" name="borderColour" value="' + '{{this}}' + '" title="{{this}}">' +
+
+                                '{{this}}' +
+                            '</label>' +
+                        '{{/each}}' +
+                    '</div>' +
+                    '<br>' +
+                '{{/if}}' +
+                '{{#if allowBackgroundColour}}' +
+                    '<label for="{{elementid}}_atto_table_backgroundcolour" class="sameline">' +
+                    '{{get_string "backgroundcolour" component}}</label>' +
+                    '<div id="{{elementid}}_atto_table_backgroundcolour"' +
+                    'class="{{CSS.BACKGROUNDCOLOUR}} {{CSS.AVAILABLECOLORS}}" size="1">' +
+                        '<label class="hideborder" for="{{../elementid}}_atto_table_backgroundcolour_-1"' +
+                        'style="background-color:transparent;color:transparent">' +
+
+                            '<input id="{{../elementid}}_atto_table_backgroundcolour_-1"' +
+                            'type="radio" name="backgroundColour" value="none" checked="checked"' +
+                            'title="{{get_string "themedefault" component}}"></input>' +
+
+                            '{{get_string "themedefault" component}}' +
+                        '</label>' +
+
+                        '{{#each availableColours}}' +
+                            '<label for="{{../elementid}}_atto_table_backgroundcolour_{{@index}}"' +
+                            'style="background-color:{{this}};color:{{this}}">' +
+
+                                '<input id="{{../elementid}}_atto_table_backgroundcolour_{{@index}}"' +
+                                'type="radio" name="backgroundColour" value="' + '{{this}}' + '" title="{{this}}">' +
+
+                                '{{this}}' +
+                            '</label>' +
+                        '{{/each}}' +
+                    '</div>' +
+                    '<br>' +
+                '{{/if}}' +
+                '{{#if allowWidth}}' +
+                    '<label for="{{elementid}}_atto_table_width" class="sameline">' +
+                    '{{get_string "width" component}}</label>' +
+                    '<input name="width" id="{{elementid}}_atto_table_width" class="{{CSS.WIDTH}}" size="8" ' +
+                        'type="number" min="0" max="100"/>' +
+                    '<label style="display: inline-block;">{{CSS.WIDTHUNIT}}</label>' +
+                    '<br>' +
+                '{{/if}}' +
+                '</fieldset>' +
+            '{{/if}}' +
+            '<div class="mdl-align">' +
+            '<br/>' +
+            '{{#if edit}}' +
+                '<button class="submit" type="submit">{{get_string "updatetable" component}}</button>' +
+            '{{/if}}' +
+            '{{#if nonedit}}' +
+                '<button class="submit" type="submit">{{get_string "createtable" component}}</button>' +
+            '{{/if}}' +
             '</div>' +
         '</form>',
     CSS = {
         CAPTION: 'caption',
+        CAPTIONPOSITION: 'captionposition',
         HEADERS: 'headers',
         ROWS: 'rows',
         COLUMNS: 'columns',
         SUBMIT: 'submit',
-        FORM: 'atto_form'
+        FORM: 'atto_form',
+        BORDERS: 'borders',
+        BORDERSIZE: 'bordersize',
+        BORDERSIZEUNIT: 'px',
+        BORDERCOLOUR: 'bordercolour',
+        BORDERSTYLE: 'borderstyle',
+        BACKGROUNDCOLOUR: 'backgroundcolour',
+        WIDTH: 'customwidth',
+        WIDTHUNIT: '%',
+        AVAILABLECOLORS: 'availablecolors',
+        COLOURROW: 'colourrow'
     },
     SELECTORS = {
         CAPTION: '.' + CSS.CAPTION,
+        CAPTIONPOSITION: '.' + CSS.CAPTIONPOSITION,
         HEADERS: '.' + CSS.HEADERS,
         ROWS: '.' + CSS.ROWS,
         COLUMNS: '.' + CSS.COLUMNS,
         SUBMIT: '.' + CSS.SUBMIT,
-        FORM: '.atto_form'
+        BORDERS: '.' + CSS.BORDERS,
+        BORDERSIZE: '.' + CSS.BORDERSIZE,
+        BORDERCOLOURS: '.' + CSS.BORDERCOLOUR + ' input[name="borderColour"]',
+        SELECTEDBORDERCOLOUR: '.' + CSS.BORDERCOLOUR + ' input[name="borderColour"]:checked',
+        BORDERSTYLE: '.' + CSS.BORDERSTYLE,
+        BACKGROUNDCOLOURS: '.' + CSS.BACKGROUNDCOLOUR + ' input[name="backgroundColour"]',
+        SELECTEDBACKGROUNDCOLOUR: '.' + CSS.BACKGROUNDCOLOUR + ' input[name="backgroundColour"]:checked',
+        FORM: '.atto_form',
+        WIDTH: '.' + CSS.WIDTH,
+        AVAILABLECOLORS: '.' + CSS.AVAILABLECOLORS
     };
 
 Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.EditorPlugin, [], {
@@ -136,7 +253,6 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
             callback: this._displayTableEditor,
             tags: 'table'
         });
-
         // Disable mozilla table controls.
         if (Y.UA.gecko) {
             document.execCommand("enableInlineTableEditing", false, false);
@@ -158,12 +274,15 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
             var dialogue = this.getDialogue({
                 headerContent: M.util.get_string('createtable', COMPONENT),
                 focusAfterHide: true,
-                focusOnShowSelector: SELECTORS.CAPTION
+                focusOnShowSelector: SELECTORS.CAPTION,
+                width: DIALOGUE.WIDTH
             });
 
             // Set the dialogue content, and then show the dialogue.
-            dialogue.set('bodyContent', this._getDialogueContent())
+            dialogue.set('bodyContent', this._getDialogueContent(false))
                     .show();
+
+            this._updateAvailableSettings();
         }
     },
 
@@ -196,29 +315,7 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
      * @return {boolean} whether or not the parameter node exists within the editor.
      */
     _stopAtContentEditableFilter: function(node) {
-        this.editor.contains(node);
-    },
-
-    /**
-     * Return the edit table dialogue content, attaching any required
-     * events.
-     *
-     * @method _getEditDialogueContent
-     * @private
-     * @return {Node} The content to place in the dialogue.
-     */
-    _getEditDialogueContent: function() {
-        var template = Y.Handlebars.compile(EDITTEMPLATE);
-        this._content = Y.Node.create(template({
-                CSS: CSS,
-                elementid: this.get('host').get('elementid'),
-                component: COMPONENT
-            }));
-
-        // Handle table setting.
-        this._content.one('.submit').on('click', this._updateTable, this);
-
-        return this._content;
+        return this.editor.contains(node);
     },
 
     /**
@@ -229,20 +326,77 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
      * @private
      * @return {Node} The content to place in the dialogue.
      */
-    _getDialogueContent: function() {
+    _getDialogueContent: function(edit) {
         var template = Y.Handlebars.compile(TEMPLATE);
+        var allowBorders = this.get('allowBorders');
+
         this._content = Y.Node.create(template({
                 CSS: CSS,
                 elementid: this.get('host').get('elementid'),
-                component: COMPONENT
+                component: COMPONENT,
+                edit: edit,
+                nonedit: !edit,
+                allowStyling: this.get('allowStyling'),
+                allowBorders: allowBorders,
+                borderStyles: this.get('borderStyles'),
+                allowBackgroundColour: this.get('allowBackgroundColour'),
+                availableColours: this.get('availableColors'),
+                allowWidth: this.get('allowWidth')
             }));
 
         // Handle table setting.
-        this._content.one('.submit').on('click', this._setTable, this);
+        if (edit) {
+            this._content.one('.submit').on('click', this._updateTable, this);
+        } else {
+            this._content.one('.submit').on('click', this._setTable, this);
+        }
+
+        if (allowBorders) {
+            this._content.one('[name="borders"]').on('change', this._updateAvailableSettings, this);
+        }
 
         return this._content;
     },
 
+    /**
+     * Disables options within the dialogue if they shouldn't be available.
+     * E.g.
+     * If borders are set to "Theme default" then the border size, style and
+     * colour options are disabled.
+     *
+     * @method _updateAvailableSettings
+     * @private
+     */
+    _updateAvailableSettings: function() {
+        var tableForm = this._content,
+            enableBorders = tableForm.one('[name="borders"]'),
+            borderStyle = tableForm.one('[name="borderstyles"]'),
+            borderSize = tableForm.one('[name="bordersize"]'),
+            borderColour = tableForm.all('[name="borderColour"]'),
+            disabledValue = 'removeAttribute';
+
+        if (!enableBorders) {
+            return;
+        }
+
+        if (enableBorders.get('value') === 'default') {
+            disabledValue = 'setAttribute';
+        }
+
+        if (borderStyle) {
+            borderStyle[disabledValue]('disabled');
+        }
+
+        if (borderSize) {
+            borderSize[disabledValue]('disabled');
+        }
+
+        if (borderColour) {
+            borderColour[disabledValue]('disabled');
+        }
+
+    },
+
     /**
      * Given the current selection, return a table cell suitable for table editing
      * purposes, i.e. the first table cell selected, or the first cell in the table
@@ -255,12 +409,13 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
     _getSuitableTableCell: function() {
         var targetcell = null,
             host = this.get('host');
+        var stopAtContentEditableFilter = Y.bind(this._stopAtContentEditableFilter, this);
 
-        host.getSelectedNodes().some(function (node) {
-            if (node.ancestor('td, th, caption', true, this._stopAtContentEditableFilter)) {
+        host.getSelectedNodes().some(function(node) {
+            if (node.ancestor('td, th, caption', true, stopAtContentEditableFilter)) {
                 targetcell = node;
 
-                var caption = node.ancestor('caption', true, this._stopAtContentEditableFilter);
+                var caption = node.ancestor('caption', true, stopAtContentEditableFilter);
                 if (caption) {
                     var table = caption.get('parentNode');
                     if (table) {
@@ -309,8 +464,15 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
      */
     _updateTable: function(e) {
         var caption,
+            captionposition,
             headers,
+            borders,
+            bordersize,
+            borderstyle,
+            bordercolour,
+            backgroundcolour,
             table,
+            width,
             captionnode;
 
         e.preventDefault();
@@ -321,20 +483,39 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
 
         // Add/update the caption.
         caption = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.CAPTION);
+        captionposition = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.CAPTIONPOSITION);
         headers = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.HEADERS);
+        borders = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.BORDERS);
+        bordersize = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.BORDERSIZE);
+        bordercolour = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.SELECTEDBORDERCOLOUR);
+        borderstyle = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.BORDERSTYLE);
+        backgroundcolour = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.SELECTEDBACKGROUNDCOLOUR);
+        width = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.WIDTH);
 
         table = this._lastTarget.ancestor('table');
+        this._setAppearance(table, {
+            width: width,
+            borders: borders,
+            borderColour: bordercolour,
+            borderSize: bordersize,
+            borderStyle: borderstyle,
+            backgroundColour: backgroundcolour
+        });
 
         captionnode = table.one('caption');
         if (!captionnode) {
-            captionnode = Y.Node.create('<caption></caption');
+            captionnode = Y.Node.create('<caption></caption>');
             table.insert(captionnode, 0);
         }
         captionnode.setHTML(caption.get('value'));
+        captionnode.setStyle('caption-side', captionposition.get('value'));
+        if (!captionnode.getAttribute('style')) {
+            captionnode.removeAttribute('style');
+        }
 
         // Add the row headers.
         if (headers.get('value') === 'rows' || headers.get('value') === 'both') {
-            table.all('tr').each(function (row) {
+            table.all('tr').each(function(row) {
                 var cells = row.all('th, td'),
                     firstCell = cells.shift(),
                     newCell;
@@ -348,7 +529,7 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                 }
 
                 // Now make sure all other cells in the row are td.
-                cells.each(function (cell) {
+                cells.each(function(cell) {
                     if (cell.get('tagName') === 'TH') {
                         newCell = this._changeNodeType(cell, 'td');
                         newCell.removeAttribute('scope');
@@ -363,7 +544,7 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                 firstRow = rows.shift(),
                 newCell;
 
-            firstRow.all('td, th').each(function (cell) {
+            firstRow.all('td, th').each(function(cell) {
                 if (cell.get('tagName') === 'TD') {
                     // Cell is a td but should be a th - change it.
                     newCell = this._changeNodeType(cell, 'th');
@@ -389,6 +570,8 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
 
             }, this);
         }
+        // Clean the HTML.
+        this.markUpdated();
     },
 
     /**
@@ -400,10 +583,17 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
      */
     _setTable: function(e) {
         var caption,
+            captionposition,
+            borders,
+            bordersize,
+            borderstyle,
+            bordercolour,
             rows,
             cols,
             headers,
             tablehtml,
+            backgroundcolour,
+            width,
             i, j;
 
         e.preventDefault();
@@ -414,18 +604,30 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         }).hide();
 
         caption = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.CAPTION);
+        captionposition = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.CAPTIONPOSITION);
+        borders = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.BORDERS);
+        bordersize = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.BORDERSIZE);
+        bordercolour = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.SELECTEDBORDERCOLOUR);
+        borderstyle = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.BORDERSTYLE);
+        backgroundcolour = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.SELECTEDBACKGROUNDCOLOUR);
         rows = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.ROWS);
         cols = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.COLUMNS);
         headers = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.HEADERS);
+        width = e.currentTarget.ancestor(SELECTORS.FORM).one(SELECTORS.WIDTH);
 
         // Set the selection.
         this.get('host').setSelection(this._currentSelection);
 
         // Note there are some spaces inserted in the cells and before and after, so that users have somewhere to click.
         var nl = "\n";
-        tablehtml = '<br/>' + nl + '<table>' + nl;
-        tablehtml += '<caption>' + Y.Escape.html(caption.get('value')) + '</caption>' + nl;
+        var tableId = Y.guid();
+        tablehtml = '<br/>' + nl + '<table id="' + tableId + '">' + nl;
 
+        var captionstyle = '';
+        if (captionposition.get('value')) {
+            captionstyle = ' style="caption-side: ' + captionposition.get('value') + '"';
+        }
+        tablehtml += '<caption' + captionstyle + '>' + Y.Escape.html(caption.get('value')) + '</caption>' + nl;
         i = 0;
         if (headers.get('value') === 'columns' || headers.get('value') === 'both') {
             i = 1;
@@ -442,7 +644,7 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
                 if (j === 0 && (headers.get('value') === 'rows' || headers.get('value') === 'both')) {
                     tablehtml += '<th scope="row"></th>' + nl;
                 } else {
-                    tablehtml += '<td></td>' + nl;
+                    tablehtml += '<td ></td>' + nl;
                 }
             }
             tablehtml += '</tr>' + nl;
@@ -452,6 +654,17 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
 
         this.get('host').insertContentAtFocusPoint(tablehtml);
 
+        var tableNode = Y.one('#' + tableId);
+        this._setAppearance(tableNode, {
+            width: width,
+            borders: borders,
+            borderColour: bordercolour,
+            borderSize: bordersize,
+            borderStyle: borderstyle,
+            backgroundColour: backgroundcolour
+        });
+        tableNode.removeAttribute('id');
+
         // Mark the content as updated.
         this.markUpdated();
     },
@@ -473,8 +686,8 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         rows.each(function(row) {
             var cells = row.all('td, th'),
                 cell = cells.item(columnindex),
-                cellprev = cells.item(columnindex-1),
-                cellnext = cells.item(columnindex+1);
+                cellprev = cells.item(columnindex - 1),
+                cellnext = cells.item(columnindex + 1);
             currentcells.push(cell);
             if (cellprev) {
                 prevcells.push(cellprev);
@@ -836,7 +1049,7 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         var cells = this._findColumnCells();
 
         // Check we have some tds in this column, and one exists to the right.
-        if ( (cells.next.size() > 0) &&
+        if ((cells.next.size() > 0) &&
                 (cells.current.size() === cells.next.size()) &&
                 (cells.current.filter('td').size() > 0)) {
             var i = 0;
@@ -869,6 +1082,102 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         this.markUpdated();
     },
 
+    /**
+     * Obtain values for the table borders
+     *
+     * @method _getBorderConfiguration
+     * @param {Node} node
+     * @private
+     * @return {Array} or {Boolean} Returns the settings, if presents, or else returns false
+     */
+    _getBorderConfiguration: function(node) {
+        // We need to make a clone of the node in order to avoid grabbing any
+        // of the computed styles from the DOM. We only want inline styles set by us.
+        var shadowNode = node.cloneNode(true);
+        var borderStyle = shadowNode.getStyle('borderStyle'),
+            borderColor = shadowNode.getStyle('borderColor'),
+            borderWidth = shadowNode.getStyle('borderWidth');
+
+        if (borderStyle || borderColor || borderWidth) {
+            var hexColour = Y.Color.toHex(borderColor);
+            var width = parseInt(borderWidth, 10);
+            return {
+                borderStyle: borderStyle,
+                borderColor: hexColour === "#" ? null : hexColour,
+                borderWidth: isNaN(width) ? null : width
+            };
+        }
+
+        return false;
+    },
+
+    /**
+     * Set the appropriate styles on the given table node according to
+     * the provided configuration.
+     *
+     * @method _setAppearance
+     * @param {Node} The table node to be modified.
+     * @param {Object} Configuration object (associative array) containing the form nodes for
+     *                 border styling.
+     * @private
+     */
+    _setAppearance: function(tableNode, configuration) {
+        var borderhex,
+            borderSizeValue,
+            borderStyleValue,
+            backgroundcolourvalue;
+
+        if (configuration.borderColour) {
+            borderhex = configuration.borderColour.get('value');
+        }
+
+        if (configuration.borderSize) {
+            borderSizeValue = configuration.borderSize.get('value');
+        }
+
+        if (configuration.borderStyle) {
+            borderStyleValue = configuration.borderStyle.get('value');
+        }
+
+        if (configuration.backgroundColour) {
+            backgroundcolourvalue = configuration.backgroundColour.get('value');
+        }
+
+        // Clear the inline border styling
+        tableNode.removeAttribute('style');
+        tableNode.all('td, th').each(function(cell) {
+            cell.removeAttribute('style');
+        }, this);
+
+        if (configuration.borders) {
+            if (configuration.borders.get('value') === 'outer') {
+                tableNode.setStyle('borderWidth', borderSizeValue + CSS.BORDERSIZEUNIT);
+                tableNode.setStyle('borderStyle', borderStyleValue);
+
+                if (borderhex !== 'none') {
+                    tableNode.setStyle('borderColor', borderhex);
+                }
+            } else if (configuration.borders.get('value') === 'all') {
+                tableNode.all('td, th').each(function(cell) {
+                    cell.setStyle('borderWidth', borderSizeValue + CSS.BORDERSIZEUNIT);
+                    cell.setStyle('borderStyle', borderStyleValue);
+
+                    if (borderhex !== 'none') {
+                        cell.setStyle('borderColor', borderhex);
+                    }
+                }, this);
+            }
+        }
+
+        if (backgroundcolourvalue !== 'none') {
+            tableNode.setStyle('backgroundColor', backgroundcolourvalue);
+        }
+
+        if (configuration.width && configuration.width.get('value')) {
+            tableNode.setStyle('width', configuration.width.get('value') + CSS.WIDTHUNIT);
+        }
+    },
+
     /**
      * Edit table (show the dialogue).
      *
@@ -879,15 +1188,25 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         var dialogue = this.getDialogue({
             headerContent: M.util.get_string('edittable', COMPONENT),
             focusAfterHide: false,
-            focusOnShowSelector: SELECTORS.CAPTION
+            focusOnShowSelector: SELECTORS.CAPTION,
+            width: DIALOGUE.WIDTH
         });
 
         // Set the dialogue content, and then show the dialogue.
-        var node = this._getEditDialogueContent(),
+        var node = this._getDialogueContent(true),
             captioninput = node.one(SELECTORS.CAPTION),
+            captionpositioninput = node.one(SELECTORS.CAPTIONPOSITION),
             headersinput = node.one(SELECTORS.HEADERS),
+            borderinput = node.one(SELECTORS.BORDERS),
+            borderstyle = node.one(SELECTORS.BORDERSTYLE),
+            bordercolours = node.all(SELECTORS.BORDERCOLOURS),
+            bordersize = node.one(SELECTORS.BORDERSIZE),
+            backgroundcolours = node.all(SELECTORS.BACKGROUNDCOLOURS),
+            width = node.one(SELECTORS.WIDTH),
             table = this._lastTarget.ancestor('table'),
-            captionnode = table.one('caption');
+            captionnode = table.one('caption'),
+            hexColour,
+            matchedInput;
 
         if (captionnode) {
             captioninput.set('value', captionnode.getHTML());
@@ -895,6 +1214,55 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
             captioninput.set('value', '');
         }
 
+        if (width && table.getStyle('width').indexOf('px') === -1) {
+            width.set('value', parseInt(table.getStyle('width'), 10));
+        }
+
+        if (captionpositioninput && captionnode && captionnode.getAttribute('style')) {
+            captionpositioninput.set('value', captionnode.getStyle('caption-side'));
+        } else {
+            // Default to none.
+            captionpositioninput.set('value', '');
+        }
+
+        if (table.getStyle('backgroundColor') && this.get('allowBackgroundColour')) {
+            hexColour = Y.Color.toHex(table.getStyle('backgroundColor'));
+            matchedInput = backgroundcolours.filter('[value="' + hexColour + '"]');
+
+            if (matchedInput) {
+                matchedInput.set("checked", true);
+            }
+        }
+
+        if (this.get('allowBorders')) {
+            var borderValue = 'default',
+                borderConfiguration = this._getBorderConfiguration(table);
+
+            if (borderConfiguration) {
+                borderValue = 'outer';
+            } else {
+                borderConfiguration = this._getBorderConfiguration(table.one('td'));
+                if (borderConfiguration) {
+                     borderValue = 'all';
+                }
+            }
+
+            if (borderConfiguration) {
+                var borderStyle = borderConfiguration.borderStyle || DEFAULT.BORDERSTYLE;
+                var borderSize = borderConfiguration.borderWidth || DEFAULT.BORDERWIDTH;
+                borderstyle.set('value', borderStyle);
+                bordersize.set('value', borderSize);
+                borderinput.set('value', borderValue);
+
+                hexColour = borderConfiguration.borderColor;
+                matchedInput = bordercolours.filter('[value="' + hexColour + '"]');
+
+                if (matchedInput) {
+                    matchedInput.set("checked", true);
+                }
+            }
+        }
+
         var headersvalue = 'columns';
         if (table.one('th[scope="row"]')) {
             headersvalue = 'rows';
@@ -904,6 +1272,7 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         }
         headersinput.set('value', headersvalue);
         dialogue.set('bodyContent', node).show();
+        this._updateAvailableSettings();
     },
 
 
@@ -945,13 +1314,11 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
      * @private
      */
     _addRowAfter: function() {
-        var rowindex = this._getRowIndex(this._lastTarget);
-
-        var tablebody = this._lastTarget.ancestor('table').one('tbody');
+        var target = this._lastTarget.ancestor('tr'),
+            tablebody = this._lastTarget.ancestor('table').one('tbody');
         if (!tablebody) {
             // Not all tables have tbody.
             tablebody = this._lastTarget.ancestor('table');
-            rowindex += 1;
         }
 
         var firstrow = tablebody.one('tr');
@@ -962,8 +1329,8 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
             // Table has no rows. Boo.
             return;
         }
-        newrow = firstrow.cloneNode(true);
-        newrow.all('th, td').each(function (tablecell) {
+        var newrow = firstrow.cloneNode(true);
+        newrow.all('th, td').each(function(tablecell) {
             if (tablecell.get('tagName') === 'TH') {
                 if (tablecell.getAttribute('scope') !== 'row') {
                     var newcell = Y.Node.create('<td></td>');
@@ -974,7 +1341,12 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
             tablecell.setHTML('&nbsp;');
         });
 
-        tablebody.insert(newrow, rowindex);
+        if (target.ancestor('thead')) {
+            target = firstrow;
+            tablebody.insert(newrow, target);
+        } else {
+            target.insert(newrow, 'after');
+        }
 
         // Clean the HTML.
         this.markUpdated();
@@ -1012,4 +1384,80 @@ Y.namespace('M.atto_table').Button = Y.Base.create('button', Y.M.editor_atto.Edi
         this.markUpdated();
     }
 
+}, {
+    ATTRS: {
+        /**
+         * Whether or not to allow borders
+         *
+         * @attribute allowBorder
+         * @type Boolean
+         */
+        allowBorders: {
+            value: true
+        },
+
+        /**
+         * What border styles to allow
+         *
+         * @attribute borderStyles
+         * @type Array
+         */
+        borderStyles: {
+            value: [
+                'none',
+                'solid',
+                'dashed',
+                'dotted'
+            ]
+        },
+
+        /**
+         * Whether or not to allow colourizing the background
+         *
+         * @attribute allowBackgroundColour
+         * @type Boolean
+         */
+        allowBackgroundColour: {
+            value: true
+        },
+
+        /**
+         * Whether or not to allow setting the table width
+         *
+         * @attribute allowWidth
+         * @type Boolean
+         */
+        allowWidth: {
+            value: true
+        },
+
+        /**
+         * Whether we allow styling
+         * @attribute allowStyling
+         * @type Boolean
+         */
+        allowStyling: {
+            readOnly: true,
+            getter: function() {
+                return this.get('allowBorders') || this.get('allowBackgroundColour') || this.get('allowWidth');
+            }
+        },
+
+        /**
+         * Available colors
+         * @attribute availableColors
+         * @type Array
+         */
+        availableColors: {
+            value: [
+                '#FFFFFF',
+                '#EF4540',
+                '#FFCF35',
+                '#98CA3E',
+                '#7D9FD3',
+                '#333333'
+            ],
+            readOnly: true
+        }
+    }
 });
