@@ -26,6 +26,7 @@
 
 defined('MOODLE_INTERNAL') || die();
 
+require_once($CFG->dirroot . '/question/type/questiontypebase.php');
 require_once($CFG->dirroot . '/question/type/questionbase.php');
 require_once($CFG->dirroot . '/question/type/numerical/question.php');
 
@@ -37,8 +38,20 @@ require_once($CFG->dirroot . '/question/type/numerical/question.php');
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class qtype_calculated extends question_type {
-    /** Regular expression that finds the formulas in content. */
-    const FORMULAS_IN_TEXT_REGEX = '~\{=([^{}]*(?:\{[^{}]+}[^{}]*)*)\}~';
+    /**
+     * @const string a placeholder is a letter, followed by almost any characters. (This should probably be restricted more.)
+     */
+    const PLACEHOLDER_REGEX_PART = '[[:alpha:]][^>} <`{"\']*';
+
+    /**
+     * @const string REGEXP for a placeholder, wrapped in its {...} delimiters, with capturing brackets around the name.
+     */
+    const PLACEHODLER_REGEX = '~\{(' . self::PLACEHOLDER_REGEX_PART . ')\}~';
+
+    /**
+     * @const string Regular expression that finds the formulas in content, with capturing brackets to get the forumlas.
+     */
+    const FORMULAS_IN_TEXT_REGEX = '~\{=([^{}]*(?:\{' . self::PLACEHOLDER_REGEX_PART . '\}[^{}]*)*)\}~';
 
     const MAX_DATASET_ITEMS = 100;
 
@@ -122,7 +135,7 @@ class qtype_calculated extends question_type {
                             $def->number_of_items = $n;
                         }
                     }
-                    $datasetdefs["1-$r->category-$r->name"] = $def;
+                    $datasetdefs["1-{$r->category}-{$r->name}"] = $def;
                 }
             }
         }
@@ -280,7 +293,7 @@ class qtype_calculated extends question_type {
                 if ($sharedatasetdefs = $DB->get_records_select(
                     'question_dataset_definitions',
                     "type = '1'
-                    AND name = ?
+                    AND " . $DB->sql_equal('name', '?') . "
                     AND category = ?
                     ORDER BY id DESC ", array($dataset->name, $question->category)
                 )) { // So there is at least one.
@@ -359,11 +372,11 @@ class qtype_calculated extends question_type {
         // See where we're coming from.
         switch($form->wizardpage) {
             case 'question':
-                require("$CFG->dirroot/question/type/calculated/datasetdefinitions.php");
+                require("{$CFG->dirroot}/question/type/calculated/datasetdefinitions.php");
                 break;
             case 'datasetdefinitions':
             case 'datasetitems':
-                require("$CFG->dirroot/question/type/calculated/datasetitems.php");
+                require("{$CFG->dirroot}/question/type/calculated/datasetitems.php");
                 break;
             default:
                 print_error('invalidwizardpage', 'question');
@@ -387,15 +400,15 @@ class qtype_calculated extends question_type {
         // See where we're coming from.
         switch($wizardnow) {
             case 'datasetdefinitions':
-                require("$CFG->dirroot/question/type/calculated/datasetdefinitions_form.php");
+                require("{$CFG->dirroot}/question/type/calculated/datasetdefinitions_form.php");
                 $mform = new question_dataset_dependent_definitions_form(
-                        "$submiturl?wizardnow=datasetdefinitions", $question);
+                        "{$submiturl}?wizardnow=datasetdefinitions", $question);
                 break;
             case 'datasetitems':
-                require("$CFG->dirroot/question/type/calculated/datasetitems_form.php");
+                require("{$CFG->dirroot}/question/type/calculated/datasetitems_form.php");
                 $regenerate = optional_param('forceregeneration', false, PARAM_BOOL);
                 $mform = new question_dataset_dependent_items_form(
-                        "$submiturl?wizardnow=datasetitems", $question, $regenerate);
+                        "{$submiturl}?wizardnow=datasetitems", $question, $regenerate);
                 break;
             default:
                 print_error('invalidwizardpage', 'question');
@@ -485,6 +498,15 @@ class qtype_calculated extends question_type {
         }
     }
 
+    /**
+     * Remove prefix #{..}# if exists.
+     * @param $name a question name,
+     * @return string the cleaned up question name.
+     */
+    public function clean_technical_prefix_from_question_name($name) {
+        return preg_replace('~#\{([^[:space:]]*)#~', '', $name);
+    }
+
     /**
      * This method prepare the $datasets in a format similar to dadatesetdefinitions_form.php
      * so that they can be saved
@@ -552,11 +574,7 @@ class qtype_calculated extends question_type {
                 AND a.category != 0
                 AND b.question = ?
            ORDER BY a.name ", array($question->id));
-        $questionname = $question->name;
-        $regs= array();
-        if (preg_match('~#\{([^[:space:]]*)#~', $questionname , $regs)) {
-            $questionname = str_replace($regs[0], '', $questionname);
-        };
+        $questionname = $this->clean_technical_prefix_from_question_name($question->name);
 
         if (!empty($categorydatasetdefs)) {
             // There is at least one with the same name.
@@ -712,22 +730,22 @@ class qtype_calculated extends question_type {
     public function custom_generator_tools_part($mform, $idx, $j) {
 
         $minmaxgrp = array();
-        $minmaxgrp[] = $mform->createElement('text', "calcmin[$idx]",
+        $minmaxgrp[] = $mform->createElement('text', "calcmin[{$idx}]",
                 get_string('calcmin', 'qtype_calculated'));
-        $minmaxgrp[] = $mform->createElement('text', "calcmax[$idx]",
+        $minmaxgrp[] = $mform->createElement('text', "calcmax[{$idx}]",
                 get_string('calcmax', 'qtype_calculated'));
         $mform->addGroup($minmaxgrp, 'minmaxgrp',
                 get_string('minmax', 'qtype_calculated'), ' - ', false);
-        $mform->setType("calcmin[$idx]", PARAM_FLOAT);
-        $mform->setType("calcmax[$idx]", PARAM_FLOAT);
+        $mform->setType("calcmin[{$idx}]", PARAM_FLOAT);
+        $mform->setType("calcmax[{$idx}]", PARAM_FLOAT);
 
         $precisionoptions = range(0, 10);
-        $mform->addElement('select', "calclength[$idx]",
+        $mform->addElement('select', "calclength[{$idx}]",
                 get_string('calclength', 'qtype_calculated'), $precisionoptions);
 
         $distriboptions = array('uniform' => get_string('uniform', 'qtype_calculated'),
                 'loguniform' => get_string('loguniform', 'qtype_calculated'));
-        $mform->addElement('select', "calcdistribution[$idx]",
+        $mform->addElement('select', "calcdistribution[{$idx}]",
                 get_string('calcdistribution', 'qtype_calculated'), $distriboptions);
     }
 
@@ -736,11 +754,11 @@ class qtype_calculated extends question_type {
         foreach ($datasetdefs as $datasetdef) {
             if (preg_match('~^(uniform|loguniform):([^:]*):([^:]*):([0-9]*)$~',
                     $datasetdef->options, $regs)) {
-                $defid = "$datasetdef->type-$datasetdef->category-$datasetdef->name";
-                $formdata["calcdistribution[$idx]"] = $regs[1];
-                $formdata["calcmin[$idx]"] = $regs[2];
-                $formdata["calcmax[$idx]"] = $regs[3];
-                $formdata["calclength[$idx]"] = $regs[4];
+                $defid = "{$datasetdef->type}-{$datasetdef->category}-{$datasetdef->name}";
+                $formdata["calcdistribution[{$idx}]"] = $regs[1];
+                $formdata["calcmin[{$idx}]"] = $regs[2];
+                $formdata["calcmax[{$idx}]"] = $regs[3];
+                $formdata["calclength[{$idx}]"] = $regs[4];
             }
             $idx++;
         }
@@ -751,7 +769,7 @@ class qtype_calculated extends question_type {
         global $OUTPUT;
         if (preg_match('~^(uniform|loguniform):([^:]*):([^:]*):([0-9]*)$~',
                 $datasetdef->options, $regs)) {
-            $defid = "$datasetdef->type-$datasetdef->category-$datasetdef->name";
+            $defid = "{$datasetdef->type}-{$datasetdef->category}-{$datasetdef->name}";
             for ($i = 0; $i<10; ++$i) {
                 $lengthoptions[$i] = get_string(($regs[1] == 'uniform'
                     ? 'decimals'
@@ -759,19 +777,19 @@ class qtype_calculated extends question_type {
             }
             $menu1 = html_writer::label(get_string('lengthoption', 'qtype_calculated'),
                 'menucalclength', false, array('class' => 'accesshide'));
-            $menu1 .= html_writer::select($lengthoptions, 'calclength[]', $regs[4], null);
+            $menu1 .= html_writer::select($lengthoptions, 'calclength[]', $regs[4], null, array('class' => 'custom-select'));
 
             $options = array('uniform' => get_string('uniformbit', 'qtype_calculated'),
                 'loguniform' => get_string('loguniformbit', 'qtype_calculated'));
             $menu2 = html_writer::label(get_string('distributionoption', 'qtype_calculated'),
                 'menucalcdistribution', false, array('class' => 'accesshide'));
-            $menu2 .= html_writer::select($options, 'calcdistribution[]', $regs[1], null);
-            return '<input type="submit" onclick="'
-                . "getElementById('addform').regenerateddefid.value='$defid'; return true;"
+            $menu2 .= html_writer::select($options, 'calcdistribution[]', $regs[1], null, array('class' => 'custom-select'));
+            return '<input type="submit" class="btn btn-secondary" onclick="'
+                . "getElementById('addform').regenerateddefid.value='{$defid}'; return true;"
                 .'" value="'. get_string('generatevalue', 'qtype_calculated') . '"/><br/>'
-                . '<input type="text" size="3" name="calcmin[]" '
-                . " value=\"$regs[2]\"/> &amp; <input name=\"calcmax[]\" "
-                . ' type="text" size="3" value="' . $regs[3] .'"/> '
+                . '<input type="text" class="form-control" size="3" name="calcmin[]" '
+                . " value=\"{$regs[2]}\"/> &amp; <input name=\"calcmax[]\" "
+                . ' type="text" class="form-control" size="3" value="' . $regs[3] .'"/> '
                 . $menu1 . '<br/>'
                 . $menu2;
         } else {
@@ -946,7 +964,7 @@ class qtype_calculated extends question_type {
                 // Fix regenerate for this datadefs.
                 $defregenerate = 0;
                 if ($synchronize &&
-                        !empty ($fromform->nextpageparam["datasetregenerate[$datasetdef->name"])) {
+                        !empty ($fromform->nextpageparam["datasetregenerate[{$datasetdef->name}"])) {
                     $defregenerate = 1;
                 } else if (!$synchronize &&
                         (($regenerate == 1 && $datasetdef->category == 0) ||$regenerate == 2)) {
@@ -1053,11 +1071,15 @@ class qtype_calculated extends question_type {
         $comment->outsidelimit = false;
         $comment->answers = array();
         // Find a default unit.
-        if (!empty($questionid) && $unit = $DB->get_record('question_numerical_units',
-                array('question' => $questionid, 'multiplier' => 1.0))) {
-            $unit = $unit->unit;
-        } else {
-            $unit = '';
+        $unit = '';
+        if (!empty($questionid)) {
+            $units = $DB->get_records('question_numerical_units',
+                array('question' => $questionid, 'multiplier' => 1.0),
+                'id ASC', '*', 0, 1);
+            if ($units) {
+                $unit = reset($units);
+                $unit = $unit->unit;
+            }
         }
 
         $answers = fullclone($answers);
@@ -1085,7 +1107,7 @@ class qtype_calculated extends question_type {
             }
             if ($answer->min === '') {
                 // This should mean that something is wrong.
-                $comment->stranswers[$key] = " $formattedanswer->answer".'<br/><br/>';
+                $comment->stranswers[$key] = " {$formattedanswer->answer}".'<br/><br/>';
             } else if ($formula === '*') {
                 $comment->stranswers[$key] = $formula . ' = ' .
                         get_string('anyvalue', 'qtype_calculated') . '<br/><br/><br/>';
@@ -1140,7 +1162,7 @@ class qtype_calculated extends question_type {
         }
         if (!$selected) {
             if ($mandatory) {
-                $selected =  "1-0-$name"; // Default.
+                $selected =  "1-0-{$name}"; // Default.
             } else {
                 $selected = '0'; // Default.
             }
@@ -1232,7 +1254,7 @@ class qtype_calculated extends question_type {
                   ORDER BY i.id";
             if ($records = $DB->get_records_sql($sql, array($questionid))) {
                 foreach ($records as $r) {
-                    $datasetdefs["$r->type-$r->category-$r->name"] = $r;
+                    $datasetdefs["{$r->type}-{$r->category}-{$r->name}"] = $r;
                 }
             }
         }
@@ -1399,7 +1421,7 @@ class qtype_calculated extends question_type {
                 // can manage to automatically take care of
                 // some possible realtime concurrence.
                 if ($olderdatasetdefs = $DB->get_records_select('question_dataset_definitions',
-                        "type = ? AND name = ? AND category = ? AND id < ?
+                        "type = ? AND " . $DB->sql_equal('name', '?') . " AND category = ? AND id < ?
                         ORDER BY id DESC",
                         array($datasetdef->type, $datasetdef->name,
                                 $datasetdef->category, $datasetdef->id))) {
@@ -1475,26 +1497,26 @@ class qtype_calculated extends question_type {
         $options['0'] = get_string($prefix.'nodataset', $langfile);
         // New question no local.
         if (!isset($form->id) || $form->id == 0) {
-            $key = "$type-0-$name";
-            $options[$key] = get_string($prefix."newlocal$type", $langfile);
+            $key = "{$type}-0-{$name}";
+            $options[$key] = get_string($prefix."newlocal{$type}", $langfile);
             $currentdatasetdef = new stdClass();
             $currentdatasetdef->type = '0';
         } else {
             // Construct question local options.
             $sql = "SELECT a.*
                 FROM {question_dataset_definitions} a, {question_datasets} b
-               WHERE a.id = b.datasetdefinition AND a.type = '1' AND b.question = ? AND a.name = ?";
+               WHERE a.id = b.datasetdefinition AND a.type = '1' AND b.question = ? AND " . $DB->sql_equal('a.name', '?');
             $currentdatasetdef = $DB->get_record_sql($sql, array($form->id, $name));
             if (!$currentdatasetdef) {
                 $currentdatasetdef = new stdClass();
                 $currentdatasetdef->type = '0';
             }
-            $key = "$type-0-$name";
+            $key = "{$type}-0-{$name}";
             if ($currentdatasetdef->type == $type
                     and $currentdatasetdef->category == 0) {
-                $options[$key] = get_string($prefix."keptlocal$type", $langfile);
+                $options[$key] = get_string($prefix."keptlocal{$type}", $langfile);
             } else {
-                $options[$key] = get_string($prefix."newlocal$type", $langfile);
+                $options[$key] = get_string($prefix."newlocal{$type}", $langfile);
             }
         }
         // Construct question category options.
@@ -1505,35 +1527,48 @@ class qtype_calculated extends question_type {
             WHERE a.id = b.datasetdefinition
             AND a.type = '1'
             AND a.category = ?
-            AND a.name = ?", array($form->category, $name));
+            AND " . $DB->sql_equal('a.name', '?'), array($form->category, $name));
         $type = 1;
-        $key = "$type-$form->category-$name";
+        $key = "{$type}-{$form->category}-{$name}";
         if (!empty($categorydatasetdefs)) {
             // There is at least one with the same name.
             if (isset($form->id) && isset($categorydatasetdefs[$form->id])) {
                 // It is already used by this question.
-                $options[$key] = get_string($prefix."keptcategory$type", $langfile);
+                $options[$key] = get_string($prefix."keptcategory{$type}", $langfile);
             } else {
-                $options[$key] = get_string($prefix."existingcategory$type", $langfile);
+                $options[$key] = get_string($prefix."existingcategory{$type}", $langfile);
             }
         } else {
-            $options[$key] = get_string($prefix."newcategory$type", $langfile);
+            $options[$key] = get_string($prefix."newcategory{$type}", $langfile);
         }
         // All done!
         return array($options, $currentdatasetdef->type
-            ? "$currentdatasetdef->type-$currentdatasetdef->category-$name"
+            ? "{$currentdatasetdef->type}-{$currentdatasetdef->category}-{$name}"
             : '');
     }
 
+    /**
+     * Find the names of all datasets mentioned in a piece of question content like the question text.
+     * @param $text the text to analyse.
+     * @return array with dataset name for both key and value.
+     */
     public function find_dataset_names($text) {
-        // Returns the possible dataset names found in the text as an array.
-        // The array has the dataset name for both key and value.
-        $datasetnames = array();
-        while (preg_match('~\\{([[:alpha:]][^>} <{"\']*)\\}~', $text, $regs)) {
-            $datasetnames[$regs[1]] = $regs[1];
-            $text = str_replace($regs[0], '', $text);
-        }
-        return $datasetnames;
+        preg_match_all(self::PLACEHODLER_REGEX, $text, $matches);
+        return array_combine($matches[1], $matches[1]);
+    }
+
+    /**
+     * Find all the formulas in a bit of text.
+     *
+     * For example, called with "What is {a} plus {b}? (Hint, it is not {={a}*{b}}.)" this
+     * returns ['{a}*{b}'].
+     *
+     * @param $text text to analyse.
+     * @return array where they keys an values are the formulas.
+     */
+    public function find_formulas($text) {
+        preg_match_all(self::FORMULAS_IN_TEXT_REGEX, $text, $matches);
+        return array_combine($matches[1], $matches[1]);
     }
 
     /**
@@ -1551,8 +1586,8 @@ class qtype_calculated extends question_type {
                      WHERE i.id = d.datasetdefinition AND i.category = ?";
             if ($records = $DB->get_records_sql($sql, array($form->category))) {
                 foreach ($records as $r) {
-                    if (!isset ($datasetdefs["$r->name"])) {
-                        $datasetdefs["$r->name"] = $r->itemcount;
+                    if (!isset ($datasetdefs["{$r->name}"])) {
+                        $datasetdefs["{$r->name}"] = $r->itemcount;
                     }
                 }
             }
@@ -1588,14 +1623,14 @@ class qtype_calculated extends question_type {
                     $sql1 = "SELECT q.*
                                FROM {question} q
                               WHERE q.id = ?";
-                    if (!isset ($datasetdefs["$r->type-$r->category-$r->name"])) {
-                        $datasetdefs["$r->type-$r->category-$r->name"] = $r;
+                    if (!isset ($datasetdefs["{$r->type}-{$r->category}-{$r->name}"])) {
+                        $datasetdefs["{$r->type}-{$r->category}-{$r->name}"] = $r;
                     }
                     if ($questionb = $DB->get_records_sql($sql1, array($r->question))) {
-                        if (!isset ($datasetdefs["$r->type-$r->category-$r->name"]->questions[$r->question])) {
-                            $datasetdefs["$r->type-$r->category-$r->name"]->questions[$r->question] = new stdClass();
+                        if (!isset ($datasetdefs["{$r->type}-{$r->category}-{$r->name}"]->questions[$r->question])) {
+                            $datasetdefs["{$r->type}-{$r->category}-{$r->name}"]->questions[$r->question] = new stdClass();
                         }
-                        $datasetdefs["$r->type-$r->category-$r->name"]->questions[
+                        $datasetdefs["{$r->type}-{$r->category}-{$r->name}"]->questions[
                                 $r->question]->name = $questionb[$r->question]->name;
                     }
                 }
@@ -1605,30 +1640,25 @@ class qtype_calculated extends question_type {
 
             $text = "<table width=\"100%\" border=\"1\"><tr>
                     <th style=\"white-space:nowrap;\" class=\"header\"
-                            scope=\"col\">$namestr</th>
+                            scope=\"col\">{$namestr}</th>
                     <th style=\"white-space:nowrap;\" class=\"header\"
-                            scope=\"col\">$rangeofvaluestr</th>
+                            scope=\"col\">{$rangeofvaluestr}</th>
                     <th style=\"white-space:nowrap;\" class=\"header\"
-                            scope=\"col\">$itemscountstr</th>
+                            scope=\"col\">{$itemscountstr}</th>
                     <th style=\"white-space:nowrap;\" class=\"header\"
-                            scope=\"col\">$questionusingstr</th>
+                            scope=\"col\">{$questionusingstr}</th>
                     </tr>";
             foreach ($datasetdefs as $datasetdef) {
                 list($distribution, $min, $max, $dec) = explode(':', $datasetdef->options, 4);
                 $text .= "<tr>
-                        <td valign=\"top\" align=\"center\">$datasetdef->name</td>
-                        <td align=\"center\" valign=\"top\">$min <strong>-</strong> $max</td>
-                        <td align=\"right\" valign=\"top\">$datasetdef->itemcount&nbsp;&nbsp;</td>
+                        <td valign=\"top\" align=\"center\">{$datasetdef->name}</td>
+                        <td align=\"center\" valign=\"top\">{$min} <strong>-</strong> $max</td>
+                        <td align=\"right\" valign=\"top\">{$datasetdef->itemcount}&nbsp;&nbsp;</td>
                         <td align=\"left\">";
                 foreach ($datasetdef->questions as $qu) {
                     // Limit the name length displayed.
-                    if (!empty($qu->name)) {
-                        $qu->name = (strlen($qu->name) > $lnamemax) ?
-                            substr($qu->name, 0, $lnamemax).'...' : $qu->name;
-                    } else {
-                        $qu->name = '';
-                    }
-                    $text .= " &nbsp;&nbsp; $qu->name <br/>";
+                    $questionname = $this->get_short_question_name($qu->name, $lnamemax);
+                    $text .= " &nbsp;&nbsp; {$questionname} <br/>";
                 }
                 $text .= "</td></tr>";
             }
@@ -1639,6 +1669,26 @@ class qtype_calculated extends question_type {
         return $text;
     }
 
+    /**
+     * This function shortens a question name if it exceeds the character limit.
+     *
+     * @param string $stringtoshorten the string to be shortened.
+     * @param int $characterlimit the character limit.
+     * @return string
+     */
+    public function get_short_question_name($stringtoshorten, $characterlimit)
+    {
+        if (!empty($stringtoshorten)) {
+            $returnstring = format_string($stringtoshorten);
+            if (strlen($returnstring) > $characterlimit) {
+                $returnstring = shorten_text($returnstring, $characterlimit, true);
+            }
+            return $returnstring;
+        } else {
+            return '';
+        }
+    }
+
     /**
      * This function build a table showing the available category shareable
      * wild cards, their name, their definition (Min, Max, Decimal) , the item count
@@ -1663,7 +1713,7 @@ class qtype_calculated extends question_type {
                      WHERE i.id = d.datasetdefinition AND i.category = ?";
             if ($records = $DB->get_records_sql($sql, array($category))) {
                 foreach ($records as $r) {
-                    $key = "$r->type-$r->category-$r->name";
+                    $key = "{$r->type}-{$r->category}-{$r->name}";
                     $sql1 = "SELECT q.*
                                FROM {question} q
                               WHERE q.id = ?";
@@ -1684,11 +1734,11 @@ class qtype_calculated extends question_type {
 
             $text  = "<table width=\"100%\" border=\"1\"><tr>
                     <th style=\"white-space:nowrap;\" class=\"header\"
-                            scope=\"col\">$namestr</th>";
+                            scope=\"col\">{$namestr}</th>";
             $text .= "<th style=\"white-space:nowrap;\" class=\"header\"
-                    scope=\"col\">$itemscountstr</th>";
+                    scope=\"col\">{$itemscountstr}</th>";
             $text .= "<th style=\"white-space:nowrap;\" class=\"header\"
-                    scope=\"col\">&nbsp;&nbsp;$questionusingstr &nbsp;&nbsp;</th>";
+                    scope=\"col\">&nbsp;&nbsp;{$questionusingstr} &nbsp;&nbsp;</th>";
             $text .= "<th style=\"white-space:nowrap;\" class=\"header\"
                     scope=\"col\">Quiz</th>";
             $text .= "<th style=\"white-space:nowrap;\" class=\"header\"
@@ -1698,23 +1748,18 @@ class qtype_calculated extends question_type {
                 $count = count($datasetdef->questions);
                 $text .= "<tr>
                         <td style=\"white-space:nowrap;\" valign=\"top\"
-                                align=\"center\" rowspan=\"$count\"> $datasetdef->name </td>
+                                align=\"center\" rowspan=\"{$count}\"> {$datasetdef->name} </td>
                         <td align=\"right\" valign=\"top\"
-                                rowspan=\"$count\">$datasetdef->itemcount</td>";
+                                rowspan=\"{$count}\">{$datasetdef->itemcount}</td>";
                 $line = 0;
                 foreach ($datasetdef->questions as $qu) {
                     // Limit the name length displayed.
-                    if (!empty($qu->name)) {
-                        $qu->name = (strlen($qu->name) > $lnamemax) ?
-                            substr($qu->name, 0, $lnamemax).'...' : $qu->name;
-                    } else {
-                        $qu->name = '';
-                    }
+                    $questionname = $this->get_short_question_name($qu->name, $lnamemax);
                     if ($line) {
                         $text .= "<tr>";
                     }
                     $line++;
-                    $text .= "<td align=\"left\" style=\"white-space:nowrap;\">$qu->name</td>";
+                    $text .= "<td align=\"left\" style=\"white-space:nowrap;\">{$questionname}</td>";
                     // TODO MDL-43779 should not have quiz-specific code here.
                     $nbofquiz = $DB->count_records('quiz_slots', array('questionid' => $qu->id));
                     $nbofattempts = $DB->count_records_sql("
@@ -1724,8 +1769,8 @@ class qtype_calculated extends question_type {
                              WHERE slot.questionid = ?
                                AND quiza.preview = 0", array($qu->id));
                     if ($nbofquiz > 0) {
-                        $text .= "<td align=\"center\">$nbofquiz</td>";
-                        $text .= "<td align=\"center\">$nbofattempts";
+                        $text .= "<td align=\"center\">{$nbofquiz}</td>";
+                        $text .= "<td align=\"center\">{$nbofattempts}";
                     } else {
                         $text .= "<td align=\"center\">0</td>";
                         $text .= "<td align=\"left\"><br/>";
@@ -1876,8 +1921,8 @@ function qtype_calculated_calculate_answer($formula, $individualdata,
             }
         } else {
             // Stick to plain numeric format.
-            $answer *= "1e$p10";
-            if (0.1 <= $answer / "1e$answerlength") {
+            $answer *= "1e{$p10}";
+            if (0.1 <= $answer / "1e{$answerlength}") {
                 $calculated->answer = $sign.$answer;
             } else {
                 // Could be an idea to add some zeros here.
@@ -1906,23 +1951,27 @@ function qtype_calculated_calculate_answer($formula, $individualdata,
  * @return string|boolean false if there are no problems. Otherwise a string error message.
  */
 function qtype_calculated_find_formula_errors($formula) {
+    foreach (['//', '/*', '#', '<?', '?>'] as $commentstart) {
+        if (strpos($formula, $commentstart) !== false) {
+            return get_string('illegalformulasyntax', 'qtype_calculated', $commentstart);
+        }
+    }
+
     // Validates the formula submitted from the question edit page.
     // Returns false if everything is alright
     // otherwise it constructs an error message.
-    // Strip away dataset names.
-    while (preg_match('~\\{[[:alpha:]][^>} <{"\']*\\}~', $formula, $regs)) {
-        $formula = str_replace($regs[0], '1', $formula);
-    }
+    // Strip away dataset names. Use 1.0 to catch illegal concatenation like {a}{b}.
+    $formula = preg_replace(qtype_calculated::PLACEHODLER_REGEX, '1.0', $formula);
 
     // Strip away empty space and lowercase it.
     $formula = strtolower(str_replace(' ', '', $formula));
 
     $safeoperatorchar = '-+/*%>:^\~<?=&|!'; /* */
-    $operatorornumber = "[$safeoperatorchar.0-9eE]";
+    $operatorornumber = "[{$safeoperatorchar}.0-9eE]";
 
-    while (preg_match("~(^|[$safeoperatorchar,(])([a-z0-9_]*)" .
-            "\\(($operatorornumber+(,$operatorornumber+((,$operatorornumber+)+)?)?)?\\)~",
-        $formula, $regs)) {
+    while (preg_match("~(^|[{$safeoperatorchar},(])([a-z0-9_]*)" .
+            "\\(({$operatorornumber}+(,{$operatorornumber}+((,{$operatorornumber}+)+)?)?)?\\)~",
+            $formula, $regs)) {
         switch ($regs[2]) {
             // Simple parenthesis.
             case '':
@@ -1976,18 +2025,18 @@ function qtype_calculated_find_formula_errors($formula) {
                 return get_string('unsupportedformulafunction', 'qtype_calculated', $regs[2]);
         }
 
-        // Exchange the function call with '1' and then check for
+        // Exchange the function call with '1.0' and then check for
         // another function call...
         if ($regs[1]) {
             // The function call is proceeded by an operator.
-            $formula = str_replace($regs[0], $regs[1] . '1', $formula);
+            $formula = str_replace($regs[0], $regs[1] . '1.0', $formula);
         } else {
             // The function call starts the formula.
-            $formula = preg_replace("~^$regs[2]\\([^)]*\\)~", '1', $formula);
+            $formula = preg_replace('~^' . preg_quote($regs[2], '~') . '\([^)]*\)~', '1.0', $formula);
         }
     }
 
-    if (preg_match("~[^$safeoperatorchar.0-9eE]+~", $formula, $regs)) {
+    if (preg_match("~[^{$safeoperatorchar}.0-9eE]+~", $formula, $regs)) {
         return get_string('illegalformulasyntax', 'qtype_calculated', $regs[0]);
     } else {
         // Formula just might be valid.
@@ -2001,10 +2050,10 @@ function qtype_calculated_find_formula_errors($formula) {
  * @return string|boolean false if there are no problems. Otherwise a string error message.
  */
 function qtype_calculated_find_formula_errors_in_text($text) {
-    preg_match_all(qtype_calculated::FORMULAS_IN_TEXT_REGEX, $text, $matches);
+    $formulas = question_bank::get_qtype('calculated')->find_formulas($text);
 
     $errors = array();
-    foreach ($matches[1] as $match) {
+    foreach ($formulas as $match) {
         $error = qtype_calculated_find_formula_errors($match);
         if ($error) {
             $errors[] = $error;
